/*******************************************************************************
 * Copyright (c) 2009, Rockwell Automation, Inc.
 * All rights reserved.
 *
 ******************************************************************************/
#define WIN32_LEAN_AND_MEAN

#include "networkconfig.h"
#include "../../cip/ciptcpipinterface.h"
#include "../../cip/cipcommon.h"
#include "../../cip/ciperror.h"
#include "../../cip/cipethernetlink.h"
#include "../../opener_api.h"
#include "../../trace.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <winsock2.h>
#include <Ws2tcpip.h>
#include <windows.h>
#include <iphlpapi.h>

#define WORKING_BUFFER_SIZE 15000
#define MAX_TRIES 3

#pragma comment(lib, "IPHLPAPI.lib")
#pragma comment(lib, "Ws2_32.lib")

int ConfigureNetworkInterfaceWin32( const char* AdapterName )
{
	PIP_ADAPTER_INFO pAdapterInfo;
	PIP_ADAPTER_INFO pAdapter = NULL;
	CipDword dwRetVal = 0;

	CipUdint ulOutBufLen = sizeof( IP_ADAPTER_INFO );
	pAdapterInfo = (IP_ADAPTER_INFO *)CipCalloc( 1, sizeof( IP_ADAPTER_INFO ) );
	if( pAdapterInfo == NULL )
	{
		OPENER_TRACE_ERR( "Error allocating memory needed to call GetAdaptersinfo\n" );
		return ERROR_OUTOFMEMORY;
	}
	// Make an initial call to GetAdaptersInfo to get
	// the necessary size into the ulOutBufLen variable
	if( GetAdaptersInfo( pAdapterInfo, &ulOutBufLen ) == ERROR_BUFFER_OVERFLOW )
	{
		CipFree( pAdapterInfo );
		pAdapterInfo = (IP_ADAPTER_INFO *)CipCalloc( ulOutBufLen, sizeof( CipUdint ) );
		if( pAdapterInfo == NULL )
		{
			OPENER_TRACE_ERR( "Error allocating memory needed to call GetAdaptersinfo\n" );
			return ERROR_OUTOFMEMORY;
		}
	}

	if( (dwRetVal = GetAdaptersInfo( pAdapterInfo, &ulOutBufLen )) == NO_ERROR )
	{
		dwRetVal = ERROR_NOINTERFACE;
		pAdapter = pAdapterInfo;
		while( pAdapter )
		{
			if( strcmp( pAdapter->AdapterName, AdapterName ) == 0 )
			{
				memcpy( &g_ethernet_link.physical_address, pAdapter->Address, 6 );

				interface_configuration_.ip_address = inet_addr(
					pAdapter->IpAddressList.IpAddress.String );
				interface_configuration_.network_mask = inet_addr(
					pAdapter->IpAddressList.IpMask.String );
				interface_configuration_.gateway = inet_addr(
					pAdapter->GatewayList.IpAddress.String );

				CipUdint host_id = ntohl( interface_configuration_.ip_address )
					& ~ntohl( interface_configuration_.network_mask );              /* see CIP spec 3-5.3 for multicast address algorithm*/
				host_id -= 1;
				host_id &= 0x3ff;

				g_multicast_configuration.starting_multicast_address = htonl(
					ntohl( inet_addr( "239.192.1.0" ) ) + (host_id << 5) );

				dwRetVal = NO_ERROR;
				break;
			}
			pAdapter = pAdapter->Next;
		}
	}
	else
	{
		OPENER_TRACE_ERR( "GetAdaptersInfo failed with error: %d\n", dwRetVal );
	}

	CipFree( pAdapterInfo );

	return NO_ERROR;
}

int ConfigureDomainNameWin32( const char* AdapterName )
{
	CipDword dwSize = 0;
	int i = 0;
	CipUdint flags = GAA_FLAG_INCLUDE_PREFIX;
	CipDword dwRetVal = NO_ERROR;
	CipUdint family = AF_INET;
	PIP_ADAPTER_ADDRESSES pAddresses = NULL;
	PIP_ADAPTER_ADDRESSES pCurrAddresses = NULL;
	IP_ADAPTER_DNS_SERVER_ADDRESS *pDnServer = NULL;
	CipUdint outBufLen = WORKING_BUFFER_SIZE; // Allocate a 15 KB buffer to start with.
	CipUdint tries = 0;

	do
	{
		pAddresses = (IP_ADAPTER_ADDRESSES *)CipCalloc( 1, outBufLen );
		if( pAddresses == NULL )
		{
			OPENER_TRACE_ERR( "Memory allocation failed for IP_ADAPTER_ADDRESSES struct\n" );
			return ERROR_OUTOFMEMORY;
		}

		dwRetVal = GetAdaptersAddresses( family, flags, NULL, pAddresses, &outBufLen );

		if( dwRetVal == ERROR_BUFFER_OVERFLOW )
		{
			CipFree( pAddresses );
			pAddresses = NULL;
		}
		else
		{
			break;
		}

		tries++;

	} while( (dwRetVal == ERROR_BUFFER_OVERFLOW) && (tries < MAX_TRIES) );

	if( dwRetVal == NO_ERROR )
	{
		dwRetVal = ERROR_NOINTERFACE;

		// If successful, output some information from the data we received
		pCurrAddresses = pAddresses;
		while( pCurrAddresses )
		{
			if( strcmp( pCurrAddresses->AdapterName, AdapterName ) == 0 )
			{
				pDnServer = pCurrAddresses->FirstDnsServerAddress;
				if( pDnServer )
				{
					for( i = 0; pDnServer != NULL; i++ )
					{
						pDnServer = pDnServer->Next;
					}
				}

				if( i != 0 )
				{

					if( NULL != interface_configuration_.domain_name.string )
					{
						/* if the string is already set to a value we have to free the resources
						 * before we can set the new value in order to avoid memory leaks.
						 */
						CipFree( interface_configuration_.domain_name.string );
					}

					char* DnsSuffix = alloca( wcstombs( NULL, pCurrAddresses->DnsSuffix, 0 ) + 1 );
					wcstombs( DnsSuffix, pCurrAddresses->DnsSuffix, 0 );

					interface_configuration_.domain_name.length = strlen( DnsSuffix );
					if( interface_configuration_.domain_name.length )
					{
						interface_configuration_.domain_name.string = (CipByte *)CipCalloc(
							interface_configuration_.domain_name.length + 1,
							sizeof( CipUsint ) );
						strcpy( interface_configuration_.domain_name.string,
							DnsSuffix );
					}
					else
					{
						interface_configuration_.domain_name.string = NULL;
					}

					interface_configuration_.name_server = ((const PSOCKADDR_IN)pCurrAddresses->FirstDnsServerAddress->Address.lpSockaddr)->sin_addr.s_addr;

					if( pCurrAddresses->FirstDnsServerAddress->Next )
						interface_configuration_.name_server_2 = ((const PSOCKADDR_IN)pCurrAddresses->FirstDnsServerAddress->Next->Address.lpSockaddr)->sin_addr.s_addr;
				}
				else { interface_configuration_.domain_name.length = 0; }

				dwRetVal = NO_ERROR;
				break;
			}

			pCurrAddresses = pCurrAddresses->Next;
		}
	}
	else
	{
		OPENER_TRACE_ERR( "Call to GetAdaptersAddresses failed with error: %d\n", dwRetVal );
	}

	CipFree( pAddresses );

	return dwRetVal;
}

void ConfigureHostName()
{
	CipWord wVersionRequested;
	WSADATA wsaData;
	int err;

	/* Use the MAKEWORD(lowbyte, highbyte) macro declared in Windef.h */
	wVersionRequested = MAKEWORD( 2, 2 );

	err = WSAStartup( wVersionRequested, &wsaData );
	if( err != 0 )
	{
		/* Tell the user that we could not find a usable */
		/* Winsock DLL.                                  */
		OPENER_TRACE_ERR( "WSAStartup failed with error: %d\n", err );
		return;
	}

	char hostname[256] = "";
	int status = 0;
	status = gethostname( hostname, sizeof( hostname ) );

	WSACleanup();

	if( NULL != hostname_.string )
	{
		/* if the string is already set to a value we have to free the resources
		 * before we can set the new value in order to avoid memory leaks.
		 */
		CipFree( hostname_.string );
	}
	hostname_.length = strlen( hostname );
	if( hostname_.length )
	{
		hostname_.string = (CipByte *)CipCalloc( hostname_.length + 1,
			sizeof( CipByte ) );
		strcpy( hostname_.string, hostname );
	}
	else
	{
		hostname_.string = NULL;
	}
}
